@@ -1,20 +1,23 @@
 import numpy as np
-import time
-
-from .constants import log
+from .constants import log, now
 
 from . import util
 from . import convex
 from . import nsphere
+from . import geometry
 from . import grouping
 from . import triangles
 from . import transformations
 
 try:
+    # scipy is a soft dependency
     from scipy import spatial
     from scipy import optimize
-except ImportError:
-    log.warning('Scipy import failed!')
+except BaseException as E:
+    # raise the exception when someone tries to use it
+    from . import exceptions
+    spatial = exceptions.ExceptionModule(E)
+    optimize = exceptions.ExceptionModule(E)
 
 
 def oriented_bounds_2D(points, qhull_options='QbB'):
@@ -29,7 +32,7 @@ def oriented_bounds_2D(points, qhull_options='QbB'):
     Returns
     ----------
     transform : (3,3) float
-      Homogenous 2D transformation matrix to move the
+      Homogeneous 2D transformation matrix to move the
       input points so that the axis aligned bounding box
       is CENTERED AT THE ORIGIN.
     rectangle : (2,) float
@@ -37,23 +40,23 @@ def oriented_bounds_2D(points, qhull_options='QbB'):
        by transform
     """
     # make sure input is a numpy array
-    points = np.asanyarray(points)
+    points = np.asanyarray(points, dtype=np.float64)
     # create a convex hull object of our points
     # 'QbB' is a qhull option which has it scale the input to unit
     # box to avoid precision issues with very large/small meshes
-    convex = spatial.ConvexHull(points,
-                                qhull_options=qhull_options)
+    convex = spatial.ConvexHull(
+        points, qhull_options=qhull_options)
 
     # (n,2,3) line segments
     hull_edges = convex.points[convex.simplices]
     # (n,2) points on the convex hull
     hull_points = convex.points[convex.vertices]
 
-    # direction of the edges of the hull polygon
-    edge_vectors = np.diff(hull_edges, axis=1).reshape((-1, 2))
+    # unit vector direction of the edges of the hull polygon
+    # filter out zero- magnitude edges via check_valid
+    edge_vectors, _ = util.unitize(np.diff(hull_edges, axis=1).reshape((-1, 2)),
+                                   check_valid=True)
 
-    # unitize vectors
-    edge_vectors /= np.linalg.norm(edge_vectors, axis=1).reshape((-1, 1))
     # create a set of perpendicular vectors
     perp_vectors = np.fliplr(edge_vectors) * [-1.0, 1.0]
 
@@ -81,7 +84,7 @@ def oriented_bounds_2D(points, qhull_options='QbB'):
     # (2,) float of smallest rectangle size
     rectangle = extents[area_min]
 
-    # find the (3,3) homogenous transformation which moves the input
+    # find the (3,3) homogeneous transformation which moves the input
     # points to have a bounding box centered at the origin
     offset = -bounds[area_min][:2] - (rectangle * .5)
     theta = np.arctan2(*edge_vectors[area_min][::-1])
@@ -102,7 +105,10 @@ def oriented_bounds_2D(points, qhull_options='QbB'):
     return transform, rectangle
 
 
-def oriented_bounds(obj, angle_digits=1, ordered=True):
+def oriented_bounds(obj,
+                    angle_digits=1,
+                    ordered=True,
+                    normal=None):
     """
     Find the oriented bounding box for a Trimesh
 
@@ -115,6 +121,10 @@ def oriented_bounds(obj, angle_digits=1, ordered=True):
        Even with less precision the returned extents will cover
        the mesh albeit with larger than minimal volume, and may
        experience substantial speedups.
+    ordered : bool
+      Return a consistent order for bounds
+    normal : None or (3,) float
+      Override search for normal on 3D meshes
 
     Returns
     ----------
@@ -152,35 +162,40 @@ def oriented_bounds(obj, angle_digits=1, ordered=True):
         raise ValueError(
             'Oriented bounds must be passed a mesh or a set of points!')
 
-    # convert face normals to spherical coordinates on the upper hemisphere
-    # the vector_hemisphere call effectivly merges negative but otherwise
-    # identical vectors
-    spherical_coords = util.vector_to_spherical(
-        util.vector_hemisphere(hull_normals))
-    # the unique_rows call on merge angles gets unique spherical directions to check
-    # we get a substantial speedup in the transformation matrix creation
-    # inside the loop by converting to angles ahead of time
-    spherical_unique = grouping.unique_rows(spherical_coords,
-                                            digits=angle_digits)[0]
-
     min_volume = np.inf
-    tic = time.time()
+    tic = now()
 
-    for spherical in spherical_coords[spherical_unique]:
-        # a matrix which will rotate each hull normal to [0,0,1]
-        to_2D = np.linalg.inv(transformations.spherical_matrix(*spherical))
+    # matrices which will rotate each hull normal to [0,0,1]
+    if normal is None:
+        # convert face normals to spherical coordinates on the upper hemisphere
+        # the vector_hemisphere call effectivly merges negative but otherwise
+        # identical vectors
+        spherical_coords = util.vector_to_spherical(
+            util.vector_hemisphere(hull_normals))
+        # the unique_rows call on merge angles gets unique spherical directions to check
+        # we get a substantial speedup in the transformation matrix creation
+        # inside the loop by converting to angles ahead of time
+        spherical_unique = grouping.unique_rows(
+            spherical_coords, digits=angle_digits)[0]
+        matrices = [np.linalg.inv(transformations.spherical_matrix(*s))
+                    for s in spherical_coords[spherical_unique]]
+    else:
+        # if explicit normal was passed use it and skip the grouping
+        matrices = [geometry.align_vectors(normal, [0, 0, 1])]
+
+    for to_2D in matrices:
         # apply the transform here
         projected = np.dot(to_2D, np.column_stack(
             (vertices, np.ones(len(vertices)))).T).T[:, :3]
 
         height = projected[:, 2].ptp()
-        rotation_2D, box = oriented_bounds_2D(projected[:, 0:2])
+        rotation_2D, box = oriented_bounds_2D(projected[:, :2])
         volume = np.product(box) * height
         if volume < min_volume:
             min_volume = volume
             min_extents = np.append(box, height)
             min_2D = to_2D.copy()
-            rotation_2D[0:2, 2] = 0.0
+            rotation_2D[:2, 2] = 0.0
             rotation_Z = transformations.planar_matrix_to_3D(rotation_2D)
 
     # combine the 2D OBB transformation with the 2D projection transform
@@ -191,7 +206,7 @@ def oriented_bounds(obj, angle_digits=1, ordered=True):
     transformed = transformations.transform_points(vertices,
                                                    to_origin)
     box_center = (transformed.min(axis=0) + transformed.ptp(axis=0) * .5)
-    to_origin[0:3, 3] = -box_center
+    to_origin[:3, 3] = -box_center
 
     # return ordered 3D extents
     if ordered:
@@ -204,7 +219,7 @@ def oriented_bounds(obj, angle_digits=1, ordered=True):
 
         # make sure transform isn't mangling triangles
         # by reversing windings on triangles
-        if np.isclose(np.trace(flip[:3, :3]), 0.0):
+        if not np.isclose(np.linalg.det(flip[:3, :3]), 1.0):
             flip[:3, :3] = np.dot(flip[:3, :3], -np.eye(3))
 
         # apply the flip to the OBB transform
@@ -213,8 +228,7 @@ def oriented_bounds(obj, angle_digits=1, ordered=True):
         min_extents = min_extents[order]
 
     log.debug('oriented_bounds checked %d vectors in %0.4fs',
-              len(spherical_unique),
-              time.time() - tic)
+              len(matrices), now() - tic)
 
     return to_origin, min_extents
 
@@ -291,6 +305,39 @@ def minimum_cylinder(obj, sample_count=6, angle_tol=.001):
             return transform, radius, height
         return volume
 
+    # we've been passed a mesh with radial symmetry
+    # use center mass and symmetry axis and go home early
+    if hasattr(obj, 'symmetry') and obj.symmetry == 'radial':
+        # find our origin
+        if obj.is_watertight:
+            # set origin to center of mass
+            origin = obj.center_mass
+        else:
+            # convex hull should be watertight
+            origin = obj.convex_hull.center_mass
+        # will align symmetry axis with Z and move origin to zero
+        to_2D = geometry.plane_transform(
+            origin=origin,
+            normal=obj.symmetry_axis)
+        # transform vertices to plane to check
+        on_plane = transformations.transform_points(
+            obj.vertices, to_2D)
+        # cylinder height is overall Z span
+        height = on_plane[:, 2].ptp()
+        # center mass is correct on plane, but position
+        # along symmetry axis may be wrong so slide it
+        slide = transformations.translation_matrix(
+            [0, 0, (height / 2.0) - on_plane[:, 2].max()])
+        to_2D = np.dot(slide, to_2D)
+        # radius is maximum radius
+        radius = (on_plane[:, :2] ** 2).sum(axis=1).max() ** 0.5
+        # save kwargs
+        result = {'height': height,
+                  'radius': radius,
+                  'transform': np.linalg.inv(to_2D)}
+        return result
+
+    # get the points on the convex hull of the result
     hull = convex.hull_points(obj)
     if not util.is_shape(hull, (-1, 3)):
         raise ValueError('Input must be reducable to 3D points!')
@@ -300,20 +347,18 @@ def minimum_cylinder(obj, sample_count=6, angle_tol=.001):
 
     # if it's rotationally symmetric the bounding cylinder
     # is almost certainly along one of the PCI vectors
-    # if hasattr(obj, 'symmetry_axis') and obj.symmetry_axis is not None:
-    #    samples = util.vector_to_spherical(obj.principal_inertia_vectors)
     if hasattr(obj, 'principal_inertia_vectors'):
         # add the principal inertia vectors if we have a mesh
         samples = np.vstack(
             (samples,
              util.vector_to_spherical(obj.principal_inertia_vectors)))
 
-    tic = [time.time()]
+    tic = [now()]
     # the projected volume at each sample
     volumes = np.array([volume_from_angles(i) for i in samples])
     # the best vector in (2,) spherical coordinates
     best = samples[volumes.argmin()]
-    tic.append(time.time())
+    tic.append(now())
 
     # since we already explored the global space, set the bounds to be
     # just around the sample that had the lowest volume
@@ -327,8 +372,8 @@ def minimum_cylinder(obj, sample_count=6, angle_tol=.001):
                           method='SLSQP',
                           bounds=bounds)
 
-    tic.append(time.time())
-    log.info('Performed search in %f and minimize in %f', *np.diff(tic))
+    tic.append(now())
+    log.debug('Performed search in %f and minimize in %f', *np.diff(tic))
 
     # actually chunk the information about the cylinder
     transform, radius, height = volume_from_angles(r['x'], return_data=True)
@@ -338,6 +383,34 @@ def minimum_cylinder(obj, sample_count=6, angle_tol=.001):
     return result
 
 
+def to_extents(bounds):
+    """
+    Convert an axis aligned bounding box to extents and
+    transform.
+
+    Parameters
+    ------------
+    bounds : (2, 3) float
+      Axis aligned bounds in space
+
+    Returns
+    ------------
+    extents : (3,) float
+      Extents of the bounding box
+    transform : (4, 4) float
+      Homogeneous transform moving extents to bounds
+    """
+    bounds = np.asanyarray(bounds, dtype=np.float64)
+    if bounds.shape != (2, 3):
+        raise ValueError('bounds must be (2, 3)')
+
+    extents = bounds.ptp(axis=0)
+    transform = np.eye(4)
+    transform[:3, 3] = bounds.mean(axis=0)
+
+    return extents, transform
+
+
 def corners(bounds):
     """
     Given a pair of axis aligned bounds, return all
