@@ -2,41 +2,71 @@
 caching.py
 -----------
 
-Functions and classes that help with tracking changes in ndarrays
-and clearing cached values based on those changes.
-"""
+Functions and classes that help with tracking changes
+in `numpy.ndarray` and clearing cached values based
+on those changes.
 
+You should really `pip install xxhash`:
+
+```
+In [23]: %timeit int(blake2b(d).hexdigest(), 16)
+102 us +/- 684 ns per loop
+
+In [24]: %timeit int(sha256(d).hexdigest(), 16)
+142 us +/- 3.73 us
+
+In [25]: %timeit xxh3_64_intdigest(d)
+3.37 us +/- 116 ns per loop
+```
+"""
+import os
+import time
+import warnings
 import numpy as np
 
-import zlib
-from hashlib import md5
-
 from functools import wraps
-
 from .constants import log
 from .util import is_sequence
 
 try:
     from collections.abc import Mapping
-except ImportError:
+except BaseException:
     from collections import Mapping
 
-# xxhash is roughly 5x faster than zlib.adler32 and
-# up to 30x faster than MD5, but is only
-# packaged in easy wheels on linux (`pip install xxhash`)
+
+# sha256 is always available
+from hashlib import sha256 as _sha256
+
+
+def sha256(item):
+    return int(_sha256(item).hexdigest(), 16)
+
+
+try:
+    # blake2b is available on Python 3 and
+    from hashlib import blake2b as _blake2b
+
+    def hash_fallback(item):
+        return int(_blake2b(item).hexdigest(), 16)
+except BaseException:
+    # fallback to sha256
+    hash_fallback = sha256
+
+# xxhash is up to 30x faster than sha256:
+# `pip install xxhash`
 try:
     # newest version of algorithm
-    from xxhash import xxh3_64_intdigest as fast_hash
+    from xxhash import xxh3_64_intdigest as hash_fast
 except BaseException:
     try:
         # older version of the algorithm
-        from xxhash import xxh64_intdigest as fast_hash
+        from xxhash import xxh64_intdigest as hash_fast
     except BaseException:
-        log.debug('falling back to MD5 hashing: ' +
-                  '`pip install xxhash`' +
-                  ' for 30x faster cache checks')
-        # use MD5 as a fallback hashing algorithm
-        def fast_hash(a): return int(md5(a).hexdigest(), 16)
+        # use hashlib as a fallback hashing library
+        log.debug('falling back to hashlib ' +
+                  'hashing: `pip install xxhash`' +
+                  'for 50x faster cache checks')
+        hash_fast = hash_fallback
 
 
 def tracked_array(array, dtype=None):
@@ -56,7 +86,7 @@ def tracked_array(array, dtype=None):
     Returns
     ------------
     tracked : TrackedArray
-      Contains input array data
+      Contains input array data.
     """
     # if someone passed us None, just create an empty array
     if array is None:
@@ -137,9 +167,9 @@ class TrackedArray(np.ndarray):
 
     Methods
     ----------
-    md5 :       str, hexadecimal MD5 of array
-    crc :       int, zlib crc32/adler32 checksum
-    fast_hash : int, CRC or xxhash.xx64
+    __hash__ : int
+      Runs the fastest available hash in this order:
+        `xxh3_64, xxh_64, blake2b, sha256`
     """
 
     def __array_finalize__(self, obj):
@@ -148,13 +178,21 @@ class TrackedArray(np.ndarray):
         This flag will be set on every change as well as
         during copies and certain types of slicing.
         """
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
+
+        self._dirty_hash = True
         if isinstance(obj, type(self)):
-            obj._modified_c = True
-            obj._modified_m = True
-            obj._modified_x = True
+            obj._dirty_hash = True
+
+    def __array_wrap__(self, out_arr, context=None):
+        """
+        Return a numpy scalar if array is 0d.
+        See https://github.com/numpy/numpy/issues/5819
+        """
+        if out_arr.ndim:
+            return np.ndarray.__array_wrap__(
+                self, out_arr, context)
+        # Match numpy's behavior and return a numpy dtype scalar
+        return out_arr[()]
 
     @property
     def mutable(self):
@@ -164,202 +202,145 @@ class TrackedArray(np.ndarray):
     def mutable(self, value):
         self.flags.writeable = value
 
-    def md5(self):
-        """
-        Return an MD5 hash of the current array.
-
-        Returns
-        -----------
-        md5 : str
-          Hexadecimal MD5 of the array
-        """
-        if self._modified_m or not hasattr(self, '_hashed_md5'):
-            if self.flags['C_CONTIGUOUS']:
-                self._hashed_md5 = md5(self).hexdigest()
-            else:
-                # the case where we have sliced our nice
-                # contiguous array into a non- contiguous block
-                # for example (note slice *after* track operation):
-                # t = util.tracked_array(np.random.random(10))[::-1]
-                self._hashed_md5 = md5(
-                    np.ascontiguousarray(self)).hexdigest()
-        self._modified_m = False
-        return self._hashed_md5
+    def hash(self):
+        warnings.warn(
+            '`array.hash()` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `array.__hash__()` or `hash(array)`',
+            DeprecationWarning)
+        return self.__hash__()
 
     def crc(self):
-        """
-        A zlib.crc32 or zlib.adler32 checksum
-        of the current data.
+        warnings.warn(
+            '`array.crc()` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `array.__hash__()` or `hash(array)`',
+            DeprecationWarning)
+        return self.__hash__()
 
-        Returns
-        -----------
-        crc : int
-          Checksum from zlib.crc32 or zlib.adler32
-        """
-        if self._modified_c or not hasattr(self, '_hashed_crc'):
-            if self.flags['C_CONTIGUOUS']:
-                self._hashed_crc = crc32(self)
-            else:
-                # the case where we have sliced our nice
-                # contiguous array into a non- contiguous block
-                # for example (note slice *after* track operation):
-                # t = util.tracked_array(np.random.random(10))[::-1]
-                contiguous = np.ascontiguousarray(self)
-                self._hashed_crc = crc32(contiguous)
-        self._modified_c = False
-        return self._hashed_crc
-
-    def fast_hash(self):
-        """
-        An xxhash.b64 hash of the array.
-
-        Returns
-        -------------
-        xx : int
-          xxhash.xxh64 hash of array.
-        """
-        # repeat the bookkeeping to get a contiguous array inside
-        # the function to avoid additional function calls
-        # these functions are called millions of times so everything helps
-        if self._modified_x or not hasattr(self, '_hashed_xx'):
-            if self.flags['C_CONTIGUOUS']:
-                self._hashed_xx = fast_hash(self)
-            else:
-                # the case where we have sliced our nice
-                # contiguous array into a non- contiguous block
-                # for example (note slice *after* track operation):
-                # t = util.tracked_array(np.random.random(10))[::-1]
-                self._hashed_xx = fast_hash(np.ascontiguousarray(self))
-        self._modified_x = False
-
-        return self._hashed_xx
+    def md5(self):
+        warnings.warn(
+            '`array.md5()` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `array.__hash__()` or `hash(array)`',
+            DeprecationWarning)
+        return self.__hash__()
 
     def __hash__(self):
         """
-        Hash is required to return an int.
+        Return a fast hash of the contents of the array.
 
         Returns
-        -----------
-        hash : int
-          Result of fast_hash
+        -------------
+        hash : long int
+          A hash of the array contents.
         """
-        return self.fast_hash()
+        # repeat the bookkeeping to get a contiguous array
+        if not self._dirty_hash and hasattr(self, '_hashed'):
+            # we have a valid hash without recomputing.
+            return self._hashed
+
+        if self.flags['C_CONTIGUOUS']:
+            hashed = hash_fast(self)
+        else:
+            # the case where we have sliced our nice
+            # contiguous array into a non- contiguous block
+            # for example (note slice *after* track operation):
+            # t = tracked_array(np.random.random(10))[::-1]
+            hashed = hash_fast(np.ascontiguousarray(self))
+
+        # assign the value and set the flag
+        self._hashed = hashed
+        self._dirty_hash = False
+
+        return hashed
 
     def __iadd__(self, *args, **kwargs):
         """
-        In- place addition.
+        In-place addition.
 
         The i* operations are in- place and modify the array,
         so we better catch all of them.
         """
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__iadd__(*args,
-                                                    **kwargs)
+
+        self._dirty_hash = True
+        return super(self.__class__, self).__iadd__(
+            *args, **kwargs)
 
     def __isub__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__isub__(*args,
-                                                    **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__isub__(
+            *args, **kwargs)
 
     def __imul__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__imul__(*args,
-                                                    **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__imul__(
+            *args, **kwargs)
 
     def __idiv__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__idiv__(*args,
-                                                    **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__idiv__(
+            *args, **kwargs)
 
     def __itruediv__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__itruediv__(*args,
-                                                        **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__itruediv__(
+            *args, **kwargs)
 
     def __imatmul__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__imatmul__(*args,
-                                                       **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__imatmul__(
+            *args, **kwargs)
 
     def __ipow__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__ipow__(*args, **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__ipow__(
+            *args, **kwargs)
 
     def __imod__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__imod__(*args, **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__imod__(
+            *args, **kwargs)
 
     def __ifloordiv__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__ifloordiv__(*args,
-                                                         **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__ifloordiv__(
+            *args, **kwargs)
 
     def __ilshift__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__ilshift__(*args,
-                                                       **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__ilshift__(
+            *args, **kwargs)
 
     def __irshift__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__irshift__(*args,
-                                                       **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__irshift__(
+            *args, **kwargs)
 
     def __iand__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__iand__(*args,
-                                                    **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__iand__(
+            *args, **kwargs)
 
     def __ixor__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__ixor__(*args,
-                                                    **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__ixor__(
+            *args, **kwargs)
 
     def __ior__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        return super(self.__class__, self).__ior__(*args,
-                                                   **kwargs)
+        self._dirty_hash = True
+        return super(self.__class__, self).__ior__(
+            *args, **kwargs)
 
     def __setitem__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        super(self.__class__, self).__setitem__(*args,
-                                                **kwargs)
+        self._dirty_hash = True
+        super(self.__class__, self).__setitem__(
+            *args, **kwargs)
 
     def __setslice__(self, *args, **kwargs):
-        self._modified_c = True
-        self._modified_m = True
-        self._modified_x = True
-        super(self.__class__, self).__setslice__(*args,
-                                                 **kwargs)
+        self._dirty_hash = True
+        super(self.__class__, self).__setslice__(
+            *args, **kwargs)
 
 
 class Cache(object):
@@ -386,7 +367,7 @@ class Cache(object):
         self.id_current = self._id_function()
         # a counter for locks
         self._lock = 0
-        # actual store for data
+        # actuSal store for data
         self.cache = {}
 
     def delete(self, key):
@@ -513,6 +494,74 @@ class Cache(object):
         self.id_current = self._id_function()
 
 
+class DiskCache(object):
+    """
+    Store results of expensive operations on disk
+    with an option to expire the results. This is used
+    to cache the multi-gigabyte test corpuses in
+    `tests/corpus.py`
+    """
+
+    def __init__(self, path, expire_days=30):
+        """
+        Create a cache on disk for storing expensive results.
+
+        Parameters
+        --------------
+        path : str
+          A writeable location on the current file path.
+        expire_days : int or float
+          How old should results be considered expired.
+
+        """
+        # store how old we allow results to be
+        self.expire_days = expire_days
+        # store the location for saving results
+        self.path = os.path.abspath(
+            os.path.expanduser(path))
+        # make sure the specified path exists
+        os.makedirs(self.path, exist_ok=True)
+
+    def get(self, key, fetch):
+        """
+        Get a key from the cache or run a calculation.
+
+        Parameters
+        -----------
+        key : str
+          Key to reference item with
+        fetch : function
+          If key isn't stored and recent run this
+          function and store its result on disk.
+        """
+        # hash the key so we have a fixed length string
+        key_hash = _sha256(key.encode('utf-8')).hexdigest()
+        # full path of result on local disk
+        path = os.path.join(self.path, key_hash)
+
+        # check to see if we can use the cache
+        if os.path.isfile(path):
+            # compute the age of the existing file in days
+            age_days = (time.time() - os.stat(path).st_mtime) / 86400.0
+            if age_days < self.expire_days:
+                # this nested condition means that
+                # the file both exists and is recent
+                # enough, so just return its contents
+                with open(path, 'rb') as f:
+                    return f.read()
+
+        log.debug('not in cache fetching: `{}`'.format(key))
+        # since we made it here our data isn't cached
+        # run the expensive function to fetch the file
+        raw = fetch()
+        # write the data so we can save it
+        with open(path, 'wb') as f:
+            f.write(raw)
+
+        # return the data
+        return raw
+
+
 class DataStore(Mapping):
     """
     A class to store multiple numpy arrays and track them all
@@ -601,7 +650,7 @@ class DataStore(Mapping):
         if not self.mutable:
             raise ValueError('DataStore is configured immutable!')
 
-        if hasattr(data, 'md5'):
+        if hasattr(data, 'hash'):
             # don't bother to re-track TrackedArray
             tracked = data
         else:
@@ -627,19 +676,18 @@ class DataStore(Mapping):
         for key, value in values.items():
             self[key] = value
 
-    def md5(self):
+    def __hash__(self):
         """
-        Get an MD5 reflecting everything in the DataStore.
+        Get a hash reflecting everything in the DataStore.
 
         Returns
         ----------
-        md5 : str
-          MD5 of data in hexadecimal
+        hash : str
+          hash of data in hexadecimal
         """
-        hasher = md5(''.join(
-            self.data[key].md5()
-            for key in sorted(self.data.keys())).encode('utf-8'))
-        return hasher.hexdigest()
+        return hash_fast(np.array(
+            [hash(v) for v in self.data.values()],
+            dtype=np.int64).tobytes())
 
     def crc(self):
         """
@@ -650,9 +698,12 @@ class DataStore(Mapping):
         crc : int
           CRC of data
         """
-        # combine with a sum of every hash
-        crc = sum(i.crc() for i in self.data.values())
-        return crc
+        warnings.warn(
+            '`array.crc()` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `array.__hash__()` or `hash(array)`',
+            DeprecationWarning)
+        return self.__hash__()
 
     def fast_hash(self):
         """
@@ -663,49 +714,18 @@ class DataStore(Mapping):
         hashed : int
           Checksum of data
         """
-        # combine every hash
-        fast = sum(i.fast_hash() for i in self.data.values())
-        return fast
+        warnings.warn(
+            '`array.fast_hash()` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `array.__hash__()` or `hash(array)`',
+            DeprecationWarning)
+        return self.__hash__()
 
+    def hash(self):
+        warnings.warn(
+            '`array.hash()` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `array.__hash__()` or `hash(array)`',
+            DeprecationWarning)
 
-def _fast_crc(count=10):
-    """
-    On certain platforms/builds zlib.adler32 is substantially
-    faster than zlib.crc32, but it is not consistent across
-    Windows/Linux/OSX.
-
-    This function runs a quick check (2ms on my machines) to
-    determine the fastest hashing function available in zlib.
-
-    Parameters
-    ------------
-    count : int
-      Number of repetitions to do on the speed trial
-
-    Returns
-    ----------
-    crc32 : function
-      Either `zlib.adler32` or `zlib.crc32`
-    """
-    import timeit
-
-    # create an array of random numbers
-    setup = 'import numpy, zlib;'
-    setup += 'd = numpy.random.random((500,3));'
-    # time crc32
-    crc32 = timeit.timeit(setup=setup,
-                          stmt='zlib.crc32(d)',
-                          number=count)
-    # time adler32
-    adler32 = timeit.timeit(setup=setup,
-                            stmt='zlib.adler32(d)',
-                            number=count)
-    if adler32 < crc32:
-        return zlib.adler32
-    else:
-        return zlib.crc32
-
-
-# get the fastest CRC32 available on the
-# current platform when trimesh is imported
-crc32 = _fast_crc()
+        return self.__hash__()
