@@ -11,14 +11,11 @@ import subprocess
 import numpy as np
 import collections
 
-from distutils.spawn import find_executable
-
 from .. import util
 from ..base import Trimesh
 
 # find the executable for binvox in PATH
-binvox_encoder = find_executable('binvox')
-
+binvox_encoder = util.which('binvox')
 Binvox = collections.namedtuple(
     'Binvox', ['rle_data', 'shape', 'translate', 'scale'])
 
