@@ -1,10 +1,12 @@
 import numpy as np
+
+import re
 from collections import deque, defaultdict
 
 try:
     # `pip install pillow`
     # optional: used for textured meshes
-    import PIL.Image as Image
+    from PIL import Image
 except BaseException as E:
     # if someone tries to use Image re-raise
     # the import error so they can debug easily
@@ -21,7 +23,6 @@ from ..constants import log, tol
 
 def load_obj(file_obj,
              resolver=None,
-             split_object=False,
              group_material=True,
              skip_materials=False,
              maintain_order=False,
@@ -37,8 +38,6 @@ def load_obj(file_obj,
     resolver : trimesh.visual.resolvers.Resolver
       Allow assets such as referenced textures and
       material files to be loaded
-    split_object : bool
-      Split meshes at each `o` declared in file
     group_material : bool
       Group faces that share the same material
       into the same mesh.
@@ -56,7 +55,6 @@ def load_obj(file_obj,
     """
     # get text as bytes or string blob
     text = file_obj.read()
-
     # if text was bytes decode into string
     text = util.decode_text(text)
 
@@ -81,7 +79,7 @@ def load_obj(file_obj,
             # turn parsed kwargs into material objects
             materials = {k: SimpleMaterial(**v)
                          for k, v in material_kwargs.items()}
-        except IOError:
+        except (IOError, TypeError):
             # usually the resolver couldn't find the asset
             log.debug('unable to load materials from: {}'.format(mtl_path))
         except BaseException:
@@ -94,8 +92,7 @@ def load_obj(file_obj,
 
     # get relevant chunks that have face data
     # in the form of (material, object, chunk)
-    face_tuples = _preprocess_faces(
-        text=text, split_object=split_object)
+    face_tuples = _preprocess_faces(text=text)
 
     # combine chunks that have the same material
     # some meshes end up with a LOT of components
@@ -134,7 +131,8 @@ def load_obj(file_obj,
         # after it finds the first newline
         # passed as arg as it's not a kwarg in python2
         face_lines = [i.split('\n', 1)[0]
-                      for i in chunk.split('\nf ')[1:]]
+                      for i in chunk.split('f ')[1:]]
+
         # then we are going to replace all slashes with spaces
         joined = ' '.join(face_lines).replace('/', ' ')
 
@@ -161,7 +159,7 @@ def load_obj(file_obj,
             # i.e. something like:
             #  '31407 31406 31408',
             #  '32303/2469 32304/2469 32305/2469',
-            log.debug('faces have mixed data, using slow fallback!')
+            log.debug('faces have mixed data: using slow fallback!')
             faces, faces_tex, faces_norm = _parse_faces_fallback(
                 face_lines)
 
@@ -169,8 +167,9 @@ def load_obj(file_obj,
             name = material
         else:
             name = current_object
-            if name is None or len(name) == 0 or name in geometry:
-                name = '{}_{}'.format(name, util.unique_id())
+
+        # ensure the name is always unique
+        name = util.unique_name(name, geometry)
 
         # try to get usable texture
         mesh = kwargs.copy()
@@ -241,8 +240,8 @@ def load_obj(file_obj,
                 # may fail on a malformed color mask
                 mesh['vertex_colors'] = vc[mask_v]
             except BaseException:
-                log.warning('failed to load vertex_colors',
-                            exc_info=True)
+                log.debug('failed to load vertex_colors',
+                          exc_info=True)
         if mask_vn is not None:
             try:
                 # may fail on a malformed mask
@@ -253,8 +252,8 @@ def load_obj(file_obj,
                         str(mesh['vertices'].shape)))
                 mesh['vertex_normals'] = normals
             except BaseException:
-                log.warning('failed to load vertex_normals',
-                            exc_info=True)
+                log.debug('failed to load vertex_normals',
+                          exc_info=True)
 
         visual = None
         if material in materials:
@@ -266,7 +265,7 @@ def load_obj(file_obj,
             visual = TextureVisuals(uv=uv)
         elif material is not None:
             # case where material is specified but not available
-            log.warning('specified material ({})  not loaded!'.format(
+            log.debug('specified material ({})  not loaded!'.format(
                 material))
         # assign the visual
         mesh['visual'] = visual
@@ -278,7 +277,8 @@ def load_obj(file_obj,
         return next(iter(geometry.values()))
 
     # add an identity transform for every geometry
-    graph = [{'geometry': k, 'frame_to': k, 'matrix': np.eye(4)}
+    graph = [{'geometry': k,
+              'frame_to': k}
              for k in geometry.keys()]
 
     # convert to scene kwargs
@@ -349,7 +349,7 @@ def parse_mtl(mtl, resolver=None):
                 material['image'] = Image.open(
                     util.wrap_as_stream(file_data))
             except BaseException:
-                log.warning('failed to load image', exc_info=True)
+                log.debug('failed to load image', exc_info=True)
 
         elif key in mapped.keys():
             try:
@@ -363,7 +363,7 @@ def parse_mtl(mtl, resolver=None):
                 # also store key by OBJ name
                 material[key] = value
             except BaseException:
-                log.warning('failed to convert color!', exc_info=True)
+                log.debug('failed to convert color!', exc_info=True)
         # pass everything as kwargs to material constructor
         elif material is not None:
             # save any other unspecified keys
@@ -432,7 +432,7 @@ def _parse_faces_vectorized(array, columns, sample_line):
             # which is vertex/texture
             faces_tex = array[:, index + 1]
         else:
-            log.warning('face lines are weird: {}'.format(
+            log.debug('face lines are weird: {}'.format(
                 sample_line))
     elif columns == 9:
         # if we have three values per vertex
@@ -491,7 +491,7 @@ def _parse_faces_fallback(lines):
              for i in range(len(split) - 2)]
             split = collect
         else:
-            log.warning(
+            log.debug(
                 'face needs more values 3>{} skipping!'.format(
                     len(split)))
             continue
@@ -670,12 +670,26 @@ def _group_by_material(face_tuples):
     return list(grouped.values())
 
 
-def _preprocess_faces(text, split_object=False):
-    # Pre-Process Face Text
-    # Rather than looking at each line in a loop we're
-    # going to split lines by directives which indicate
-    # a new mesh, specifically 'usemtl' and 'o' keys
-    # search for materials, objects, faces, or groups
+def _preprocess_faces(text):
+    """
+    Pre-Process Face Text
+
+    Rather than looking at each line in a loop we're
+    going to split lines by directives which indicate
+    a new mesh, specifically 'usemtl' and 'o' keys
+    search for materials, objects, faces, or groups
+
+    Parameters
+    ------------
+    text : str
+      Raw file
+
+    Returns
+    ------------
+    triple : (n, 3) tuple
+      Tuples of (material, object, data-chunk)
+    """
+    # see which chunk is relevant
     starters = ['\nusemtl ', '\no ', '\nf ', '\ng ', '\ns ']
     f_start = len(text)
     # first index of material, object, face, group, or smoother
@@ -702,57 +716,39 @@ def _preprocess_faces(text, split_object=False):
         # check to make sure our subsetting didn't miss any faces
         assert f_chunk.count('\nf ') == text.count('\nf ')
 
-    # start with undefined objects and material
-    current_object = None
-    current_material = None
-    # where we're going to store result tuples
-    # containing (material, object, face lines)
+    # two things cause new meshes to be created:
+    # objects and materials
+    # re.finditer was faster than find in a loop
+    # find the index of every material change
+    idx_mtl = np.array([m.start(0) for m in re.finditer(
+        'usemtl ', f_chunk)], dtype=int)
+    # find the index of every new object
+    idx_obj = np.array([m.start(0) for m in re.finditer(
+        '\no ', f_chunk)], dtype=int)
+
+    # find all the indexes where we want to split
+    splits = np.unique(np.concatenate((
+        [0, len(f_chunk)],
+        idx_mtl,
+        idx_obj)))
+
+    # track the current material and object ID
+    current_obj = None
+    current_mtl = None
+    # store (material, object, face lines)
     face_tuples = []
 
-    # two things cause new meshes to be created: objects and materials
-    # first divide faces into groups split by material and objects
-    # face chunks using different materials will be treated
-    # as different meshes
-    for m_chunk in f_chunk.split('\nusemtl '):
-        # if empty continue
-        if len(m_chunk) == 0:
-            continue
-
-        # find the first newline in the chunk
-        # everything before it will be the usemtl direction
-        new_line = m_chunk.find('\n')
-        # if the file contained no materials it will start with a newline
-        if new_line == 0:
-            current_material = None
-        else:
-            # remove internal double spaces because why wouldn't that be OK
-            current_material = ' '.join(m_chunk[:new_line].strip().split())
-
-        # material chunk contains multiple objects
-        if split_object:
-            o_split = m_chunk.split('\no ')
-        else:
-            o_split = [m_chunk]
-        if len(o_split) > 1:
-            for o_chunk in o_split:
-                # set the object label
-                current_object = o_chunk[:o_chunk.find('\n')].strip()
-                # find the first face in the chunk
-                f_idx = o_chunk.find('\nf ')
-                # if we have any faces append it to our search tuple
-                if f_idx >= 0:
-                    face_tuples.append(
-                        (current_material,
-                         current_object,
-                         o_chunk[f_idx:]))
-        else:
-            # if there are any faces in this chunk add them
-            f_idx = m_chunk.find('\nf ')
-            if f_idx >= 0:
-                face_tuples.append(
-                    (current_material,
-                     current_object,
-                     m_chunk[f_idx:]))
+    for start, end in zip(splits[:-1], splits[1:]):
+        # ensure there's always a trailing newline
+        chunk = f_chunk[start:end].strip() + '\n'
+        if chunk.startswith('o '):
+            current_obj, chunk = chunk.split('\n', 1)
+            current_obj = current_obj[2:].strip()
+        elif chunk.startswith('usemtl'):
+            current_mtl, chunk = chunk.split('\n', 1)
+            current_mtl = current_mtl[6:].strip()
+        if 'f ' in chunk:
+            face_tuples.append((current_mtl, current_obj, chunk))
     return face_tuples
 
 
@@ -764,6 +760,7 @@ def export_obj(mesh,
                write_texture=True,
                resolver=None,
                digits=8,
+               mtl_name=None,
                header='https://github.com/mikedh/trimesh'):
     """
     Export a mesh as a Wavefront OBJ file.
@@ -788,6 +785,8 @@ def export_obj(mesh,
       Resolver which can write referenced text objects
     digits : int
       Number of digits to include for floating point
+    mtl_name : None or str
+      If passed, the file name of the MTL file.
     header : str or None
       Header string for top of file or None for no header.
 
@@ -816,27 +815,29 @@ def export_obj(mesh,
     else:
         raise ValueError('must be Trimesh or Scene!')
 
+    # collect lines to export
     objects = deque([])
-
+    # keep track of the number of each export element
     counts = {'v': 0, 'vn': 0, 'vt': 0}
+    # collect materials as we go
+    materials = {}
+    materials_name = set()
 
-    tex_data = None
-
-    for mesh in meshes:
+    for current in meshes:
         # we are going to reference face_formats with this
         face_type = ['v']
         # OBJ includes vertex color as RGB elements on the same line
         if (include_color and
-            mesh.visual.kind in ['vertex', 'face'] and
-                len(mesh.visual.vertex_colors)):
+            current.visual.kind in ['vertex', 'face'] and
+                len(current.visual.vertex_colors)):
 
             # create a stacked blob with position and color
             v_blob = np.column_stack((
-                mesh.vertices,
-                to_float(mesh.visual.vertex_colors[:, :3])))
+                current.vertices,
+                to_float(current.visual.vertex_colors[:, :3])))
         else:
             # otherwise just export vertices
-            v_blob = mesh.vertices
+            v_blob = current.vertices
 
             # add the first vertex key and convert the array
         # add the vertices
@@ -847,11 +848,11 @@ def export_obj(mesh,
                 row_delim='\nv ',
                 digits=digits)])
         # only include vertex normals if they're already stored
-        if include_normals and mesh._cache.cache.get('vertex_normals') is not None:
+        if include_normals and current._cache.cache.get('vertex_normals') is not None:
 
             try:
                 converted = util.array_to_string(
-                    mesh.vertex_normals,
+                    current.vertex_normals,
                     col_delim=' ',
                     row_delim='\nvn ',
                     digits=digits)
@@ -862,18 +863,33 @@ def export_obj(mesh,
                 log.debug('failed to convert vertex normals',
                           exc_info=True)
 
-        if include_texture and hasattr(mesh.visual, 'uv'):
+        # collect materials into a dict
+        if include_texture and hasattr(current.visual, 'uv'):
             try:
-                material = mesh.visual.material
+                # get a SimpleMaterial
+                material = current.visual.material
                 if hasattr(material, 'to_simple'):
                     material = material.to_simple()
-                (tex_data,
-                 tex_name,
-                 mtl_name) = material.to_obj()
 
-                if len(np.shape(getattr(mesh.visual, 'uv', None))) == 2:
+                # hash the material to avoid duplicates
+                hashed = hash(material)
+                if hashed not in materials:
+                    # get a unique name for the material
+                    name = util.unique_name(
+                        material.name, materials_name)
+                    # add the name to our collection
+                    materials_name.add(name)
+                    # convert material to an OBJ MTL
+                    materials[hashed] = material.to_obj(
+                        name=name)
+
+                # get the name of the current material as-stored
+                tex_name = materials[hashed][1]
+
+                # export the UV coordinates
+                if len(np.shape(getattr(current.visual, 'uv', None))) == 2:
                     converted = util.array_to_string(
-                        mesh.visual.uv,
+                        current.visual.uv,
                         col_delim=' ',
                         row_delim='\nvt ',
                         digits=digits)
@@ -881,9 +897,6 @@ def export_obj(mesh,
                     face_type.append('vt')
                     # add the uv coordinates
                     export.append('vt ' + converted)
-
-                # add the reference to the MTL file
-                objects.appendleft('mtllib {}'.format(mtl_name))
                 # add the directive to use the exported material
                 export.appendleft('usemtl {}'.format(tex_name))
             except BaseException:
@@ -893,22 +906,56 @@ def export_obj(mesh,
         # the format for a single vertex reference of a face
         face_format = face_formats[tuple(face_type)]
         # add the exported faces to the export if available
-        if hasattr(mesh, 'faces'):
+        if hasattr(current, 'faces'):
             export.append('f ' + util.array_to_string(
-                mesh.faces + 1 + counts['v'],
+                current.faces + 1 + counts['v'],
                 col_delim=' ',
                 row_delim='\nf ',
                 value_format=face_format))
         # offset our vertex position
-        counts['v'] += len(mesh.vertices)
+        counts['v'] += len(current.vertices)
 
         # add object name if found in metadata
-        if 'name' in mesh.metadata:
+        if 'name' in current.metadata:
             export.appendleft(
-                '\no {}'.format(mesh.metadata['name']))
+                '\no {}'.format(current.metadata['name']))
         # add this object
         objects.append('\n'.join(export))
 
+    # combine materials
+    if len(materials) > 0:
+        # collect text for a single mtllib file
+        mtl_lib = []
+        # collect files like images to write
+        mtl_data = {}
+        # now loop through: keys are garbage hash
+        # values are (data, name)
+        for data, name in materials.values():
+            for file_name, file_data in data.items():
+                if file_name.lower().endswith('.mtl'):
+                    # collect mtl lines into single file
+                    mtl_lib.append(file_data)
+                elif file_name not in mtl_data:
+                    # things like images
+                    mtl_data[file_name] = file_data
+                else:
+                    log.warning('not writing {}'.format(file_name))
+
+        if mtl_name is None:
+            # if no name passed set a default
+            mtl_name = 'material.mtl'
+
+        # prepend a header to the MTL text if requested
+        if header is not None:
+            prepend = '# {}\n\n'.format(header).encode('utf-8')
+        else:
+            prepend = b''
+
+        # save the material data
+        mtl_data[mtl_name] = prepend + b'\n\n'.join(mtl_lib)
+        # add the reference to the MTL file
+        objects.appendleft('mtllib {}'.format(mtl_name))
+
     if header is not None:
         # add a created-with header to the top of the file
         objects.appendleft('# {}'.format(header))
@@ -916,13 +963,13 @@ def export_obj(mesh,
     text = '\n'.join(objects)
 
     # if we have a resolver and have asked to write texture
-    if write_texture and resolver is not None and tex_data is not None:
+    if write_texture and resolver is not None and len(materials) > 0:
         # not all resolvers have a write method
-        [resolver.write(k, v) for k, v in tex_data.items()]
+        [resolver.write(k, v) for k, v in mtl_data.items()]
 
     # if we exported texture it changes returned values
     if return_texture:
-        return text, tex_data
+        return text, mtl_data
 
     return text
 
