@@ -1,14 +1,12 @@
 import os
 import platform
 
+from ..util import which
 from .generic import MeshScript
 from ..constants import log
 
-from distutils.spawn import find_executable
-
 # start the search with the user's PATH
 _search_path = os.environ['PATH']
-
 # add additional search locations on windows
 if platform.system() == 'Windows':
     # split existing path by delimiter
@@ -23,11 +21,10 @@ if platform.system() == 'Darwin':
     _search_path.append('/Applications/OpenSCAD.app/Contents/MacOS')
     _search_path = ':'.join(_search_path)
     log.debug('searching for scad in: %s', _search_path)
-
 # try to find the SCAD executable by name
-_scad_executable = find_executable('openscad', path=_search_path)
+_scad_executable = which('openscad', path=_search_path)
 if _scad_executable is None:
-    _scad_executable = find_executable('OpenSCAD', path=_search_path)
+    _scad_executable = which('OpenSCAD', path=_search_path)
 exists = _scad_executable is not None
 
 
@@ -47,7 +44,9 @@ def interface_scad(meshes, script, debug=False, **kwargs):
     """
     if not exists:
         raise ValueError('No SCAD available!')
-    with MeshScript(meshes=meshes, script=script, debug=debug) as scad:
+    # OFF is a simple text format that references vertices by-index
+    # making it slightly preferable to STL for this kind of exchange duty
+    with MeshScript(meshes=meshes, script=script, debug=debug, exchange='off') as scad:
         result = scad.run(_scad_executable + ' $SCRIPT -o $MESH_POST')
     return result
 
